@@ -33,17 +33,12 @@
 
 #define DEBUG_PARSE_PACKET 1
 
-/* Maximum length of packets to avoid excessive memory allocation.  */
-#define MAX_KEY_PACKET_LENGTH     (256 * 1024)
-#define MAX_UID_PACKET_LENGTH     (  2 * 1024)
-#define MAX_COMMENT_PACKET_LENGTH ( 64 * 1024)
-#define MAX_ATTR_PACKET_LENGTH    ( 16 * 1024*1024)
 
-/* Constants to allocate static MPI arrays.  */
-#define PUBKEY_MAX_NPKEY  OPENPGP_MAX_NPKEY
-#define PUBKEY_MAX_NSKEY  OPENPGP_MAX_NSKEY
-#define PUBKEY_MAX_NSIG   OPENPGP_MAX_NSIG
-#define PUBKEY_MAX_NENC   OPENPGP_MAX_NENC
+/* Constants to allocate static MPI arrays. */
+#define PUBKEY_MAX_NPKEY  5
+#define PUBKEY_MAX_NSKEY  7
+#define PUBKEY_MAX_NSIG   2
+#define PUBKEY_MAX_NENC   2
 
 /* Usage flags */
 #define PUBKEY_USAGE_SIG     GCRY_PK_USAGE_SIGN  /* Good for signatures. */
@@ -56,23 +51,6 @@
       | GCRY_PK_USAGE_AUTH | GCRY_PK_USAGE_UNKN) >= 256
 # error Please choose another value for PUBKEY_USAGE_NONE
 #endif
-#define PUBKEY_USAGE_GROUP   512                 /* Group flag.  */
-#define PUBKEY_USAGE_RENC    1024                /* Restricted encryption.  */
-#define PUBKEY_USAGE_TIME    2048                /* Timestamp use.  */
-
-
-/* The usage bits which can be derived from the algo.  */
-#define PUBKEY_USAGE_BASIC_MASK  (PUBKEY_USAGE_SIG|PUBKEY_USAGE_ENC\
-                                  |PUBKEY_USAGE_CERT|PUBKEY_USAGE_AUTH)
-
-/* The usage bits which define encryption.  */
-#define PUBKEY_USAGE_XENC_MASK  (PUBKEY_USAGE_ENC | PUBKEY_USAGE_RENC)
-
-/* Bitflags to convey hints on what kind of signature is created.  */
-#define SIGNHINT_KEYSIG  1
-#define SIGNHINT_SELFSIG 2
-#define SIGNHINT_ADSK    4
-
 
 /* Helper macros.  */
 #define is_RSA(a)     ((a)==PUBKEY_ALGO_RSA || (a)==PUBKEY_ALGO_RSA_E \
@@ -94,8 +72,7 @@ typedef enum {
     PREFTYPE_NONE = 0,
     PREFTYPE_SYM = 1,
     PREFTYPE_HASH = 2,
-    PREFTYPE_ZIP = 3,
-    PREFTYPE_AEAD = 4
+    PREFTYPE_ZIP = 3
 } preftype_t;
 
 typedef struct {
@@ -116,14 +93,12 @@ typedef struct
 /* A symmetric-key encrypted session key packet as defined in RFC
    4880, Section 5.3.  All fields are serialized.  */
 typedef struct {
-  /* We support version 4 (rfc4880) and 5 (rfc4880bis).  */
+  /* RFC 4880: this must be 4.  */
   byte version;
-  /* The cipher algorithm used to encrypt the session key.  Note that
-   * this may be different from the algorithm that is used to encrypt
-   * bulk data.  */
+  /* The cipher algorithm used to encrypt the session key.  (This may
+     be different from the algorithm that is used to encrypt the SED
+     packet.)  */
   byte cipher_algo;
-  /* The AEAD algorithm or 0 for CFB encryption.  */
-  byte aead_algo;
   /* The string-to-key specifier.  */
   STRING2KEY s2k;
   /* The length of SESKEY in bytes or 0 if this packet does not
@@ -131,8 +106,7 @@ typedef struct {
      S2K function on the password is the session key. See RFC 4880,
      Section 5.3.)  */
   byte seskeylen;
-  /* The session key as encrypted by the S2K specifier.  For AEAD this
-   * includes the nonce and the authentication tag.  */
+  /* The session key as encrypted by the S2K specifier.  */
   byte seskey[1];
 } PKT_symkey_enc;
 
@@ -153,17 +127,6 @@ typedef struct {
 } PKT_pubkey_enc;
 
 
-/* An object to build a list of public-key encrypted session key.  */
-struct pubkey_enc_list
-{
-  struct pubkey_enc_list *next;
-  u32 keyid[2];
-  int pubkey_algo;
-  int result;
-  gcry_mpi_t data[PUBKEY_MAX_NENC];
-};
-
-
 /* A one-pass signature packet as defined in RFC 4880, Section
    5.4.  All fields are serialized.  */
 typedef struct {
@@ -203,13 +166,24 @@ struct revocation_key {
   byte class;
   /* The public-key algorithm ID.  */
   byte algid;
-  /* The length of the fingerprint.  */
-  byte fprlen;
   /* The fingerprint of the authorized key.  */
   byte fpr[MAX_FINGERPRINT_LEN];
 };
 
 
+/* Object to keep information about a PKA DNS record. */
+typedef struct
+{
+  int valid;    /* An actual PKA record exists for EMAIL. */
+  int checked;  /* Set to true if the FPR has been checked against the
+                   actual key. */
+  char *uri;    /* Malloced string with the URI. NULL if the URI is
+                   not available.*/
+  unsigned char fpr[20]; /* The fingerprint as stored in the PKA RR. */
+  char email[1];/* The email address from the notation data. */
+} pka_info_t;
+
+
 /* A signature packet (RFC 4880, Section 5.2).  Only a subset of these
    fields are directly serialized (these are marked as such); the rest
    are read from the subpackets, which are not synthesized when
@@ -228,8 +202,8 @@ typedef struct
     unsigned policy_url:1;  /* At least one policy URL is present */
     unsigned notation:1;    /* At least one notation is present */
     unsigned pref_ks:1;     /* At least one preferred keyserver is present */
-    unsigned key_block:1;   /* A key block subpacket is present.  */
     unsigned expired:1;
+    unsigned pka_tried:1;   /* Set if we tried to retrieve the PKA record. */
   } flags;
   /* The key that allegedly generated this signature.  (Directly
      serialized in v3 sigs; for v4 sigs, this must be explicitly added
@@ -256,7 +230,8 @@ typedef struct
   const byte *trust_regexp;
   struct revocation_key *revkey;
   int numrevkeys;
-  int help_counter;          /* Used internally bu some functions.  */
+  pka_info_t *pka_info;      /* Malloced PKA data or NULL if not
+                                available.  See also flags.pka_tried. */
   char *signers_uid;         /* Malloced value of the SIGNERS_UID
                               * subpacket or NULL.  This string has
                               * already been sanitized.  */
@@ -301,7 +276,7 @@ typedef struct
   /* The length of ATTRIB_DATA.  */
   unsigned long attrib_len;
   byte *namehash;
-  u16 help_key_usage;
+  int help_key_usage;
   u32 help_key_expire;
   int help_full_count;
   int help_marginal_count;
@@ -315,13 +290,9 @@ typedef struct
   struct
   {
     unsigned int mdc:1;
-    unsigned int aead:1;
     unsigned int ks_modify:1;
     unsigned int compacted:1;
     unsigned int primary:2; /* 2 if set via the primary flag, 1 if calculated */
-                            /* Note that this flag is set in a
-                             * keyblock at max for one User ID and for
-                             * one User Attribute per keyblock.  */
     unsigned int revoked:1;
     unsigned int expired:1;
   } flags;
@@ -400,10 +371,10 @@ typedef struct
      when serializing.  (Serialized.)  */
   byte    version;
   byte    selfsigversion; /* highest version of all of the self-sigs */
-  byte    fprlen;         /* 0 or length of FPR.  */
-  byte    pubkey_algo;    /* The public key algorithm.  (PGP format)  */
-  u16     pubkey_usage;   /* carries the usage info.            */
-  u16     req_usage;      /* hack to pass a request to getkey() */
+  /* The public key algorithm.  (Serialized.)  */
+  byte    pubkey_algo;
+  byte    pubkey_usage;   /* for now only used to pass it to getkey() */
+  byte    req_usage;      /* hack to pass a request to getkey() */
   u32     has_expired;    /* set to the expiration date if expired */
   /* keyid of the primary key.  Never access this value directly.
      Instead, use pk_main_keyid().  */
@@ -411,13 +382,10 @@ typedef struct
   /* keyid of this key.  Never access this value directly!  Instead,
      use pk_keyid().  */
   u32     keyid[2];
-  /* Fingerprint of the key.  Only valid if FPRLEN is not 0.  */
-  byte    fpr[MAX_FINGERPRINT_LEN];
   prefitem_t *prefs;      /* list of preferences (may be NULL) */
   struct
   {
     unsigned int mdc:1;           /* MDC feature set.  */
-    unsigned int aead:1;          /* AEAD feature set.  */
     unsigned int disabled_valid:1;/* The next flag is valid.  */
     unsigned int disabled:1;      /* The key has been disabled.  */
     unsigned int primary:1;       /* This is a primary key.  */
@@ -488,13 +456,12 @@ typedef struct {
 typedef struct {
   /* Remaining length of encrypted data. */
   u32  len;
-  /* When encrypting in CFB mode, the first block size bytes of data
-   * are random data and the following 2 bytes are copies of the last
-   * two bytes of the random data (RFC 4880, Section 5.7).  This
-   * provides a simple check that the key is correct.  EXTRALEN is the
-   * size of this extra data or, in AEAD mode, the length of the
-   * headers and the tags.  This is used by build_packet when writing
-   * out the packet's header. */
+  /* When encrypting, the first block size bytes of data are random
+     data and the following 2 bytes are copies of the last two bytes
+     of the random data (RFC 4880, Section 5.7).  This provides a
+     simple check that the key is correct.  extralen is the size of
+     this extra data.  This is used by build_packet when writing out
+     the packet's header. */
   int  extralen;
   /* Whether the serialized version of the packet used / should use
      the new format.  */
@@ -504,17 +471,8 @@ typedef struct {
      Note: this is ignored when encrypting.  */
   byte is_partial;
   /* If 0, MDC is disabled.  Otherwise, the MDC method that was used
-     (only DIGEST_ALGO_SHA1 has ever been defined).  */
+     (currently, only DIGEST_ALGO_SHA1 is supported).  */
   byte mdc_method;
-  /* If 0, AEAD is not used.  Otherwise, the used AEAD algorithm.
-   * MDC_METHOD (above) shall be zero if AEAD is used.  */
-  byte aead_algo;
-  /* The cipher algo for/from the AEAD packet.  0 for other encryption
-   * packets. */
-  byte cipher_algo;
-  /* The chunk byte from the AEAD packet.  */
-  byte chunkbyte;
-
   /* An iobuf holding the data to be decrypted.  (This is not used for
      encryption!)  */
   iobuf_t buf;
@@ -646,14 +604,10 @@ int proc_signature_packets_by_fd (ctrl_t ctrl,
 int proc_encryption_packets (ctrl_t ctrl, void *ctx, iobuf_t a);
 int list_packets( iobuf_t a );
 
-const byte *issuer_fpr_raw (PKT_signature *sig, size_t *r_len);
 char *issuer_fpr_string (PKT_signature *sig);
 
 /*-- parse-packet.c --*/
 
-
-void register_known_notation (const char *string);
-
 /* Sets the packet list mode to MODE (i.e., whether we are dumping a
    packet or not).  Returns the current mode.  This allows for
    temporarily suspending dumping by doing the following:
@@ -779,61 +733,58 @@ int skip_some_packets (iobuf_t inp, unsigned int n);
 int parse_signature( iobuf_t inp, int pkttype, unsigned long pktlen,
 		     PKT_signature *sig );
 
-/* Given a signature packet, either:
- *
- *   - test whether there are any subpackets with the critical bit set
- *     that we don't understand,
- *
- *   - list the subpackets, or,
- *
- *   - find a subpacket with a specific type.
- *
- * The WANT_HASHED flag indicates that the hashed area shall be
- * considered.
- *
- * REQTYPE indicates the type of operation.
- *
- * If REQTYPE is SIGSUBPKT_TEST_CRITICAL, then this function checks
- * whether there are any subpackets that have the critical bit and
- * which GnuPG cannot handle.  If GnuPG understands all subpackets
- * whose critical bit is set, then this function returns simply
- * returns SUBPKTS.  If there is a subpacket whose critical bit is set
- * and which GnuPG does not understand, then this function returns
- * NULL and, if START is not NULL, sets *START to the 1-based index of
- * the subpacket that violates the constraint.
- *
- * If REQTYPE is SIGSUBPKT_LIST_HASHED or SIGSUBPKT_LIST_UNHASHED, the
- * packets are dumped.  Note: if REQTYPE is SIGSUBPKT_LIST_HASHED,
- * this function does not check whether the hash is correct; this is
- * merely an indication of the section that the subpackets came from.
- *
- * If REQTYPE is anything else, then this function interprets the
- * values as a subpacket type and looks for the first subpacket with
- * that type.  If such a packet is found, *CRITICAL (if not NULL) is
- * set if the critical bit was set, *RET_N is set to the offset of the
- * subpacket's content within the SUBPKTS buffer, *START is set to the
- * 1-based index of the subpacket within the buffer, and returns
- * &SUBPKTS[*RET_N].
- *
- * *START is the number of initial subpackets to not consider.  Thus,
- * if *START is 2, then the first 2 subpackets are ignored.
- */
-const byte *enum_sig_subpkt (PKT_signature *sig, int want_hashed,
-                             sigsubpkttype_t reqtype,
-                             size_t *ret_n, int *start, int *critical );
+/* Given a subpacket area (typically either PKT_signature.hashed or
+   PKT_signature.unhashed), either:
+
+     - test whether there are any subpackets with the critical bit set
+       that we don't understand,
+
+     - list the subpackets, or,
+
+     - find a subpacket with a specific type.
+
+   REQTYPE indicates the type of operation.
+
+   If REQTYPE is SIGSUBPKT_TEST_CRITICAL, then this function checks
+   whether there are any subpackets that have the critical bit and
+   which GnuPG cannot handle.  If GnuPG understands all subpackets
+   whose critical bit is set, then this function returns simply
+   returns SUBPKTS.  If there is a subpacket whose critical bit is set
+   and which GnuPG does not understand, then this function returns
+   NULL and, if START is not NULL, sets *START to the 1-based index of
+   the subpacket that violates the constraint.
+
+   If REQTYPE is SIGSUBPKT_LIST_HASHED or SIGSUBPKT_LIST_UNHASHED, the
+   packets are dumped.  Note: if REQTYPE is SIGSUBPKT_LIST_HASHED,
+   this function does not check whether the hash is correct; this is
+   merely an indication of the section that the subpackets came from.
+
+   If REQTYPE is anything else, then this function interprets the
+   values as a subpacket type and looks for the first subpacket with
+   that type.  If such a packet is found, *CRITICAL (if not NULL) is
+   set if the critical bit was set, *RET_N is set to the offset of the
+   subpacket's content within the SUBPKTS buffer, *START is set to the
+   1-based index of the subpacket within the buffer, and returns
+   &SUBPKTS[*RET_N].
+
+   *START is the number of initial subpackets to not consider.  Thus,
+   if *START is 2, then the first 2 subpackets are ignored.  */
+const byte *enum_sig_subpkt ( const subpktarea_t *subpkts,
+                              sigsubpkttype_t reqtype,
+                              size_t *ret_n, int *start, int *critical );
 
 /* Shorthand for:
- *
- *    enum_sig_subpkt (sig, want_hashed, reqtype, ret_n, NULL, NULL);
- */
-const byte *parse_sig_subpkt (PKT_signature *sig, int want_hashed,
-                              sigsubpkttype_t reqtype,
-                              size_t *ret_n );
+
+     enum_sig_subpkt (buffer, reqtype, ret_n, NULL, NULL); */
+const byte *parse_sig_subpkt ( const subpktarea_t *buffer,
+                               sigsubpkttype_t reqtype,
+                               size_t *ret_n );
 
 /* This calls parse_sig_subpkt first on the hashed signature area in
- * SIG and then, if that returns NULL, calls parse_sig_subpkt on the
- * unhashed subpacket area in SIG.  */
-const byte *parse_sig_subpkt2 (PKT_signature *sig, sigsubpkttype_t reqtype);
+   SIG and then, if that returns NULL, calls parse_sig_subpkt on the
+   unhashed subpacket area in SIG.  */
+const byte *parse_sig_subpkt2 ( PKT_signature *sig,
+                                sigsubpkttype_t reqtype);
 
 /* Returns whether the N byte large buffer BUFFER is sufficient to
    hold a subpacket of type TYPE.  Note: the buffer refers to the
@@ -866,16 +817,14 @@ PACKET *create_gpg_control ( ctrlpkttype_t type,
                              size_t datalen );
 
 /*-- build-packet.c --*/
-gpg_error_t build_keyblock_image (kbnode_t keyblock, iobuf_t *r_iobuf);
 int build_packet (iobuf_t out, PACKET *pkt);
 gpg_error_t build_packet_and_meta (iobuf_t out, PACKET *pkt);
-gpg_error_t gpg_mpi_write (iobuf_t out, gcry_mpi_t a, unsigned int *t_nwritten);
+gpg_error_t gpg_mpi_write (iobuf_t out, gcry_mpi_t a);
 gpg_error_t gpg_mpi_write_nohdr (iobuf_t out, gcry_mpi_t a);
 u32 calc_packet_length( PACKET *pkt );
 void build_sig_subpkt( PKT_signature *sig, sigsubpkttype_t type,
 			const byte *buffer, size_t buflen );
-void build_sig_subpkt_from_sig (PKT_signature *sig, PKT_public_key *pksk,
-                                unsigned int signhints);
+void build_sig_subpkt_from_sig (PKT_signature *sig, PKT_public_key *pksk);
 int  delete_sig_subpkt(subpktarea_t *buffer, sigsubpkttype_t type );
 void build_attribute_subpkt(PKT_user_id *uid,byte type,
 			    const void *buf,u32 buflen,
@@ -897,7 +846,6 @@ void free_user_id( PKT_user_id *uid );
 void free_comment( PKT_comment *rem );
 void free_packet (PACKET *pkt, parse_packet_ctx_t parsectx);
 prefitem_t *copy_prefs (const prefitem_t *prefs);
-PKT_public_key *copy_public_key_basics (PKT_public_key *d, PKT_public_key *s);
 PKT_public_key *copy_public_key( PKT_public_key *d, PKT_public_key *s );
 PKT_signature *copy_signature( PKT_signature *d, PKT_signature *s );
 PKT_user_id *scopy_user_id (PKT_user_id *sd );
@@ -915,26 +863,23 @@ int check_signature (ctrl_t ctrl, PKT_signature *sig, gcry_md_hd_t digest);
  * R_PK is not NULL, it is stored at RET_PK.)  DIGEST contains a
  * valid hash context that already includes the signed data.  This
  * function adds the relevant meta-data to the hash before finalizing
- * it and verifying the signature.  FOCRED_PK is usually NULL. */
+ * it and verifying the signature.  */
 gpg_error_t check_signature2 (ctrl_t ctrl,
                               PKT_signature *sig, gcry_md_hd_t digest,
-                              const void *extrahash, size_t extrahashlen,
-                              PKT_public_key *forced_pk,
                               u32 *r_expiredate, int *r_expired, int *r_revoked,
                               PKT_public_key **r_pk);
 
 
 /*-- pubkey-enc.c --*/
-gpg_error_t get_session_key (ctrl_t ctrl, struct pubkey_enc_list *k, DEK *dek);
+gpg_error_t get_session_key (ctrl_t ctrl, PKT_pubkey_enc *k, DEK *dek);
 gpg_error_t get_override_session_key (DEK *dek, const char *string);
 
 /*-- compress.c --*/
 int handle_compressed (ctrl_t ctrl, void *ctx, PKT_compressed *cd,
 		       int (*callback)(iobuf_t, void *), void *passthru );
 
-/*-- decrypt-data.c --*/
-int decrypt_data (ctrl_t ctrl, void *ctx, PKT_encrypted *ed, DEK *dek,
-                  int *compliance_error);
+/*-- encr-data.c --*/
+int decrypt_data (ctrl_t ctrl, void *ctx, PKT_encrypted *ed, DEK *dek );
 
 /*-- plaintext.c --*/
 gpg_error_t get_output_file (const byte *embedded_name, int embedded_namelen,
@@ -948,7 +893,7 @@ int ask_for_detached_datafile( gcry_md_hd_t md, gcry_md_hd_t md2,
 int make_keysig_packet (ctrl_t ctrl,
                         PKT_signature **ret_sig, PKT_public_key *pk,
 			PKT_user_id *uid, PKT_public_key *subpk,
-			PKT_public_key *pksk, int sigclass,
+			PKT_public_key *pksk, int sigclass, int digest_algo,
 			u32 timestamp, u32 duration,
 			int (*mksubpkt)(PKT_signature *, void *),
 			void *opaque,
